#~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*
#					FUNO: ANOVA NO-PARAMTRICA PARA MEDIDAS REPETIDAS 
											  	
#
# Descrio e detalhes: ver documentao
#
#
# Autor: Soane Mota dos Santos <soanems@gmail.com> e Julio M. Singer <jmsinger@ime.usp.br>
#
# Referncias:
# BRUNNER E., DOMHOF, S., and LANGER, F. (2002). Nonparametric Analysis of Longitudinal Data in Factorial Experiments, 
# Wiley, New York.
#
#
# Exemplo com clula vazia:
#
# emptyCell<-rbind(c("G1","T4"),c("G2","T6")) #Clulas vazias no grupo G1 tempo T4 e no grupo G2 e tempo T6. 
# C.EC<-list(C1,...,Ck)
#
# EmtyCell <- ANOVA.NPar(var=Resp, time=Tempo, group=Grupo, subject=ID, 
#		  time.order = c("T1",...,"Tt"), group.order = c("G1",...,"Gg"), 
#	  	  var.name = "Resposta", time.name = "Tempo", group.name = "Grupo", 
#	        emptyCell=emptyCell, contrast=C.EC, rounds=4, alpha = 0.05,
#		  graphs = c(IndvPlot=TRUE,IntPlotMean=TRUE,IntPlotRTE=TRUE))
#
#~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*
require(sciplot)
require(MASS)

ANOVA.NPar <- function(var, time, group = NULL, subject, time.order = NULL, group.order = NULL, 
				  var.name = NULL, time.name = "Time", group.name = NULL, emptyCell=NULL, 
				  contrast=NULL, rounds=4, alpha = 0.05, graphs = c(IndvPlot=TRUE,IntPlotMean=TRUE,IntPlotRTE=TRUE)){ 

	variavel_S <- var
	tempo_S <- time
	grupo_S <-group
	sujeito_S <- subject
	ordem.tempo_S <- time.order
	ordem.grupo_S <- group.order
	if(!is.null(group.order)& is.null(group.name)) group.name<-"Group"

    if (is.null(var) || is.null(time) || is.null(subject)) 
        stop("At least one of the input parameters (var, time, or subject) is not found.")

	if(!is.null(emptyCell)){

		#Conferindo as clulas vazias declaradas com as encontradas#
		aux1<-table(group,time) # Se zero = celula vazia
		aux1<-data.frame(aux1)
		aux2<-aux1[which(aux1$Freq==0),]
		dtim<-dim(aux2)
		dEC<-dim(emptyCell)
		if(dtim[1]!=dEC[1]) 
			stop("Ferifique a dimenso das Clulas Vazias!!!")
		
		xx<-0
		yy<-0

		for(i in 1:dtim[1]){
			xx[i]<-paste(aux2[i,1],aux2[i,2])
			yy[i]<-paste(emptyCell[i,1],emptyCell[i,2])
		}
		if(!all(sort(xx)==sort(yy))) 
			stop("As clulas Vazias declaradas so diferente das encontradas.")

		dad<-data.frame(subject,group,time,var)
		dad$time<-as.factor(dad$time)
		dad$group<-as.factor(dad$group)
		g_l<-length(levels(dad$group))
		g_t<-length(levels(dad$time))
		n.EC<-0

		# calcula o tamanho de cada grupo
		for(i in 1:g_l){
			n.EC[i]<-length(table(dad$subject[dad$group==levels(dad$group)[i]]))
		}

		dim_l<-sum(max(n.EC)*g_t*g_l)
		dad<-dad[order(dad$group,dad$subject,dad$time),] 	#ordena o banco

		dad.aux<-dad
		dad.aux$subject<- as.numeric(dad.aux$subject)
		dad.aux$time<- as.numeric(dad.aux$time)
		dad.aux$group <- as.numeric(dad.aux$group)
		
		for(i in 1:g_l){
			v1a<-unique(dad.aux[dad.aux$group==i,1])
			dad.aux$sub1[dad.aux$group==i]<- dad.aux$subject[dad.aux$group==i]-min(v1a)+1
		}

		dad1<-matrix(NA,nrow=dim_l,ncol=5)
		dad1<-data.frame(dad1)
		names(dad1)<-c("ID","ID.Aux","Grupo","Tempo","Resp")

		# Monta Grupo
		dad1$Grupo<-rep(1:g_l,each=max(n.EC)*g_t)

		# monta tempo
		dad1$Tempo<-1:g_t

		# montando u.a.
		for(i in 1:g_l){
			aux_g<-1:max(n.EC)
			dad1$ID.Aux<-rep(aux_g,each=g_t)
		}

		# montando u.a. original
		ID.EC<- max(table(dad1$Tempo))
		for(i in 1:g_l){
			aux_g<-1:ID.EC
			dad1$ID<-rep(aux_g,each=g_t)
		}
	
		dad1$Tempo.O<-factor(dad1$Tempo,labels=levels(dad$time))
		dad1$Grupo.O<-factor(dad1$Grupo,labels=levels(dad$group))

		#Completa Banco com as clulas vazias declaradas!!!
		aux3<-paste(dad.aux$sub1,dad.aux$group,dad.aux$time)
		aux4<-paste(dad1$ID.Aux,dad1$Grupo,dad1$Tempo)
		
		for(i in 1:length(aux3)){
			for(k in 1:length(aux4)){
				if(aux3[i]==aux4[k]){
					dad1$Resp[k]<-dad$var[i]
				}
			}
		}

		dad1$Trat<-paste(dad1$Grupo, "_",dad1$Tempo, sep = "")
		dad1$Trat<-factor(dad1$Trat)
		subject <- dad1$ID.Aux
		time <- dad1$Trat
		group <- NULL
		var <- dad1$Resp
		time.order <- unique(dad1$Trat)
		group.order <- NULL
		time.name <- time.name #aqui!!!
		group.name <- NULL

		gg<-length(unique(dad1$Grupo))
		tt<-length(unique(dad1$Tempo))

		variavel_S <- dad1$Resp
		tempo_S <- dad1$Tempo.O
		grupo_S <-dad1$Grupo.O
		sujeito_S <- dad1$ID

	}

    sublen <- length(subject)
    varlen <- length(var)
    timlen <- length(time)

    if ((sublen != varlen) || (sublen != timlen)) 
        stop("At least one of the input parameters (var, time, or subject) has a different length.")

    rte.twoWay <- function(group, time, indx, rscore) {
        a <- nlevels(group)
        t <- nlevels(time)
        tab <- t(matrix(mean.factor(rscore, group:time, indx), t, a))
        rankmean.g <- as.vector(apply(tab, 1, mean, na.rm = TRUE))
        rankmean.y <- as.vector(apply(tab, 2, mean, na.rm = TRUE))
        rankmean.gy <- mean.factor(rscore, group:time, indx)
        RankMean <- c(rankmean.g, rankmean.y, rankmean.gy)
        Nobs <- c(tapply(indx, group, sum, na.rm = TRUE), tapply(indx, time, sum, na.rm = TRUE), 
		tapply(indx, group:time, sum, na.rm = TRUE))
        RTE <- (1/sum(indx)) * (RankMean - 0.5)
        out <- data.frame(RankMeans = RankMean, Nobs = Nobs, RTE = RTE)
        levels(group) <- paste(group.name, glevel, sep = "")
        levels(time) <- paste(time.name, tlevel, sep = "")
        row.names(out) <- c(levels(group), levels(time), levels(group:time))
        return(out)
    }
    wald.test.twoWay <- function(group, time, subject, rscore, ind, ni, roundS) {
        n <- sum(ni)
        N <- sum(ind)
        a <- nlevels(group)
        t <- nlevels(time)
        V <- V(group, time, subject, rscore, ind, a, t, ni)$V
        R <- V(group, time, subject, rscore, ind, a, t, ni)$R
        tab <- t(matrix(mean.factor(rscore, group:time, ind), t, a))
        t.mean <- as.vector(apply(tab, 2, mean, na.rm = TRUE))
        p <- (R - 0.5)/N
        Pa <- I(a) - (1/a) * J(a)
        Pt <- I(t) - (1/t) * J(t)
        Pat <- kronecker(Pa, Pt)
        cpg <- sqrt(n) * Pa %*% kronecker(I(a), (1/t) * t(one(t))) %*% p

        Sigma <- kronecker(Pa, (1/t) * t(one(t))) %*% V %*% kronecker(Pa,(1/t) * one(t))
        cvc <- Pa %*% Sigma %*% Pa
        Q.a <- t(cpg) %*% ginv(cvc) %*% cpg
        df.a <- tr(cvc %*% ginv(cvc))
        if (!is.na(Q.a) && (Q.a > 0)) 
            pval.a <- round(1 - pchisq(Q.a, df.a), Inf)
        else pval.a <- NA

        A <- c(W = round(Q.a, roundS), df = df.a, pval = round(pval.a, roundS))

        S <- kronecker((1/a) * t(one(a)), I(t)) %*% V %*% kronecker((1/a) * one(a), I(t))
        cpt <- Pt %*% t.mean
        cvc <- Pt %*% S %*% Pt
        Q.t <- (n/N^2) * t(cpt) %*% ginv(cvc) %*% cpt
        df.t <- tr(cvc %*% ginv(cvc))
        if (!is.na(Q.t) && (Q.t > 0)) 
            pval.t <- round(1 - pchisq(Q.t, df.t), Inf)
        else pval.t <- NA

        T <- c(round(Q.t, roundS), df.t, round(pval.t, roundS))

        Cat <- kronecker(Pa, Pt)
        cvc <- Cat %*% V %*% t(Cat)
        Q.at <- (n/N^2) * t(Cat %*% R) %*% ginv(cvc) %*% Cat %*% R
        df.at <- tr(cvc %*% ginv(cvc))
        if (!is.na(Q.at) && (Q.at > 0)) 
            pval.at <- round(1 - pchisq(Q.at, df.at), Inf)
        else pval.at <- NA

        AT <- c(round(Q.at, roundS), df.at, round(pval.at, roundS))

        out.w <- rbind(A, T, AT)
        colnames(out.w) <- c("Statistic", "df", "p-value")
        rownames(out.w) <- c(group.name, time.name, paste(group.name, 
            ":", time.name, sep = ""))
        out <- out.w
    }
    anova.test.twoWay <- function(group, time, subject, rscore, ind, a, ni, roundS) {
        group <- as.factor(group)
        time <- as.factor(time)
        t <- nlevels(time)
        t.mean <- apply(t(matrix(mean.factor(rscore, group:time, ind), t, a)), 2, mean, na.rm = TRUE)
        n <- sum(ni)
        N <- sum(ind)
        V <- V(group, time, subject, rscore, ind, a, t, ni)$V
        R <- V(group, time, subject, rscore, ind, a, t, ni)$R
        p <- (R - 0.5)/N
        Pa <- I(a) - (1/a) * J(a)
        Pt <- I(t) - (1/t) * J(t)
        Pat <- kronecker(Pa, Pt)
        cpg <- sqrt(n) * Pa %*% kronecker(I(a), (1/t) * t(one(t))) %*% p
        Sigma <- kronecker(Pa, (1/t) * t(one(t))) %*% V %*% kronecker(Pa,(1/t) * one(t))
        F.a <- t(cpg) %*% cpg/sum(diag(Pa %*% Sigma))
        df1.a <- sum(diag(Pa %*% Sigma))^2/sum(diag((Pa %*% Sigma) %*% (Pa %*% Sigma)))
        if ((!is.na(F.a)) && (!is.na(df1.a)) && (F.a > 0) && (df1.a > 0)) 
            pval.a <- 1 - pchisq(F.a * df1.a, df1.a)
        else pval.a <- NA

        A <- c(B = round(F.a,roundS), df = round(df1.a,2), pval = round(pval.a, roundS))

        S <- kronecker((1/a) * t(one(a)), I(t)) %*% V %*% kronecker((1/a) * 
            one(a), I(t))
        cpt <- Pt %*% t.mean
        F.t <- (n/N^2) * (t(cpt) %*% cpt)/sum(diag(Pt %*% S))
        df1.t <- sum(diag(Pt %*% S))^2/sum(diag(Pt %*% S %*% Pt %*% S))
        if ((!is.na(F.t)) && (!is.na(df1.t)) && (F.t > 0) && (df1.t > 0)) 
            pval.t <- 1 - pchisq(F.t * df1.t, df1.t)
        else pval.t <- NA

        T <- c(round(F.t,roundS), round(df1.t,2), round(pval.t, roundS))

        F.at <- n * t(p) %*% Pat %*% p/sum(diag(Pat %*% V))
        df1.at <- sum(diag(Pat %*% V))^2/sum(diag(Pat %*% V %*% 
            Pat %*% V))
        if ((!is.na(F.at)) && (!is.na(df1.at)) && (F.at > 0) && 
            (df1.at > 0)) 
            pval.at <- 1 - pchisq(F.at * df1.at, df1.at)
        else pval.at <- NA

        AT <- c(round(F.at,roundS), round(df1.at,2),round(pval.at, roundS))

        out.box <- rbind(A, T, AT)
        colnames(out.box) <- c("Statistic", "df", "p-value")
        rownames(out.box) <- c(group.name, time.name, paste(group.name, 
            ":", time.name, sep = ""))
        df1 <- df(V, a, t, ni, ind)$df1
        df2 <- df(V, a, t, ni, ind)$df2
        if ((!is.na(F.a)) && (!is.na(df1)) && (!is.na(df2)) && 
            (F.a > 0) && (df1 > 0) && (df2 > 0)) 
            pval.mb <- 1 - pf(F.a, df1, df2)
        else pval.mb <- NA
        A <- rbind(round(c(B = F.a, df1 = df1, df2 = df2, pval = pval.mb), 
            Inf))
        colnames(A) <- c("Statistic", "df1", "df2", "p-value")
        rownames(A) <- c(group.name)
        out <- out.box
        return(out)
    }
    one <- function(d) return(matrix(1, d, 1))
    I <- function(d) {
        junk <- rep(1, length = d)
        junk <- diag(junk)
        return(junk)
    }
    J <- function(d1, d2 = d1) return(matrix(1, d1, d2))
    count.subj <- function(group, subject) {
        group <- as.factor(group)
        table <- table(group, subject)
        n <- matrix(0, nlevels(group), 1)
        for (i in 1:nlevels(group)) n[i] <- length(colnames(table)[table[i, 
            ] > 0])
        return(n)
    }

    Vi <- function(data) {
        time <- factor(data$time)
        subj <- factor(data$subj)
        srank <- data$rscore
        indx <- data$indx
        t <- nlevels(time)
        n0 <- nlevels(subj)
        srank.m <- matrix(tapply(srank, subj:time, sum, na.rm = TRUE), t, n0)
        ind.m <- matrix(tapply(indx, subj:time, sum, na.rm = TRUE), t, n0)
        junk <- srank.m * ind.m
        t.junk <- t(ind.m)
        R <- apply(junk, 1, sum, na.rm = TRUE)/apply(ind.m, 1, sum, na.rm = TRUE)
        junk1 <- t(junk - R)
        V <- matrix(0, t, t)
        for (i in 1:t) {
            for (j in 1:t) {
                ls <- sum(t.junk[, i])
                if (i == j) 
                  V[i, i] <- sum(t.junk[, i] * junk1[, i]^2)/(ls * 
                    (ls - 1))
                else {
                  lss <- sum(t.junk[, i] * t.junk[, j])
                  kss <- (ls - 1) * (sum(t.junk[, j]) - 1) + 
                    lss - 1
                  V[i, j] <- sum(t.junk[, i] * t.junk[, j] * 
                    junk1[, i] * junk1[, j])/kss
                }
            }
        }
        out <- list(R = R, V = V)
        return(out)
    }

##matriz de variancia e covariancia twoway
    V<- function(group, time, subj, rscore, indx, a, t, ni) {
        group <- as.factor(group)
        subj <- as.factor(subj)
        time <- as.factor(time)
        data <- data.frame(group, time, subj, rscore, indx)
        n <- sum(ni)
        N <- sum(indx)
        split.d <- split(data, data$group)
        V <- matrix(0, a * t, a * t)
        R <- matrix(0, a * t, 1)
        i <- 1
        k <- 1
        while (i <= a) {
            V[k:(i * t), k:(i * t)] <- Vi(split.d[[i]])$V
            R[k:(i * t), 1] <- Vi(split.d[[i]])$R
            k <- i * t + 1
            i <- i + 1
        }
        V <- V * n/N^2
        out <- list(V = V, R = R)
    }

    mean.factor <- function(y, fact, indx) {
        tab <- tapply(y, fact, sum, na.rm = TRUE)
        ind <- tapply(indx, fact, sum, na.rm = TRUE)
        return(c(tab/ind))
    }

    df <- function(V, a, t, ni, ind) {
        Pa <- I(a) - (1/a) * J(a)
        Pt <- I(t) - (1/t) * J(t)
        c <- kronecker(Pa, (1/t) * t(one(t)))
        tt <- t(c) %*% ginv(c %*% t(c)) %*% c
        tem1 <- tt %*% V
        df1 <- (tr(tem1))^2/tr(tem1 %*% tem1)
        dpr <- Pa * I(a)
        mat <- kronecker(I(a), (1/t) * t(one(t)))
        va <- mat %*% V %*% t(mat)
        lambda <- solve(diag(c(ni)) - I(a))
        tem1 <- (tr(dpr %*% va))^2
        tem2 <- tr(dpr %*% dpr %*% va %*% va %*% lambda)
        df2 <- tem1/tem2
        return(list(df1 = df1, df2 = df2))
    }

    tr <- function(x) return(sum(diag(x)))

#matriz de covariancia e variancia oneway

    covr <- function(mvar, mind, rmean, t) {
        V <- matrix(0, t, t)
        lam <- apply(mind, 2, sum, na.rm = TRUE)
        mvarvec <- c(mvar)
        mvarvec <- replace(mvarvec, which(is.na(mvarvec)), 0)
        mvar <- matrix(mvarvec, ncol = t)
        for (i in 1:t) {
            V[i, i] <- t(mind)[i, ] %*% ((mvar[, i] - rmean[i])^2)/(lam[i] * 
                (lam[i] - 1))
            if (i < t) {
                j <- i + 1
                for (j in (i + 1):t) {
                  Lam <- t(mind)[i, ] %*% mind[, j]
                  Lam2 <- sum(t(mind)[i, ] * t(mind)[j, ])
                  K <- (lam[i] - 1) * (lam[j] - 1) + Lam - 1
                  V[i, j] <- (1/K) * (t(mind)[i, ] * t(mind)[j, 
                    ]) %*% ((mvar[, i] - rmean[i]) * (mvar[, 
                    j] - rmean[j]))
                  V[j, i] <- V[i, j]
                }
            }
        }
        return(V)
    }

	Statistics <- function(C, V, p, n, g){
		#Calculating Wald-type statistics (Cp=0)
		Maux0 <- ginv(C%*%V%*%t(C)) 			
		Maux1 <- p%*%t(C)%*%Maux0%*%C%*%t(p)	
		QWald <- n*Maux1					
		pvalue.QWald <- 1-pchisq(QWald, g)
 	
		#Calculating ANOVA-type Statistic
		M <- t(C)%*%ginv(C%*%t(C))%*%C		
		MV <- M%*%V						
		trMV <- sum(diag(MV))				
		MVMV <- MV%*%MV					
		MAux2 <- p%*%M%*%t(p) 
		trMVMV <- sum(diag(MVMV))
		F <- n*MAux2/trMV		
		f_hat <- trMV^2/trMVMV	
		pvalue.ANOVA <- 1-pf(F, f_hat, 10000)

		st<-matrix(0,2,3)
 		st[1,1]<-round(QWald,4)
 		st[1,2]<-round(g,0.0)
 		st[1,3]<-round(pvalue.QWald,4)
 		st[2,1]<-round(F,3)
 		st[2,2]<-round(f_hat,2)
 		st[2,3]<-round(pvalue.ANOVA,4)
 		colnames(st)<-c("Statistic","g.l.","p-value")
 		rownames(st)<-c("QWald","QANOVA")
		list(Contrast=round(C,2),st)

	}

###################################################################################################################
    tlevel <- unique(time)
    slevel <- unique(subject)
    t <- length(tlevel)
    s <- length(slevel)
    N <- length(var)

    if (!is.null(time.order)) {
        tlevel <- time.order
        tlevel2 <- unique(time)
        if (length(tlevel) != length(tlevel2)) 
            stop("Length of the time.order vector (", length(tlevel), 
                ") \n\t\tis not equal to the levels of time vector (", 
                length(tlevel2), ").", sep = "")
        if (mean(sort(tlevel) == sort(tlevel2)) != 1) 
            stop("Elements in the time.order vector is different from the levels specified in the time vector.", sep = "")
    }

    sortvector <- double(length(var))
    newtime <- double(length(var))
    newsubject <- double(length(var))


    if(is.null(group)) { 
 	   for (i in 1:length(var)) {
      	  row <- which(subject[i] == slevel)
              col <- which(time[i] == tlevel)
        	  newsubject[i] <- row
        	  newtime[i] <- col
              sortvector[((col - 1) * s + row)] <- i
   	   }
	   subject <- newsubject[sortvector]
    	   var <- var[sortvector]
  	   time <- newtime[sortvector]
   	   #time <- factor(time)
   	   ind <- 1 - is.na(var)
	   Nmiss <- N - sum(ind)
   	   n <- length(unique(subject))
         rankvar <- rank(var,  na.last = "keep")
         rankvar <- rankvar * ind
         junk1 <- sapply(split(rankvar, factor(time):factor(subject)), sum)
         junk2 <- sapply(split(ind, factor(time):factor(subject)), sum)
         mean.time.subj <- matrix(junk1/junk2, n, t)
         mind <- is.finite(mean.time.subj)
         rankmean <- apply(mean.time.subj, 2, function(x) {
        	mean(x, na.rm = TRUE)
    	   })
    	   Nobs <- sapply(split(ind, factor(time)), sum)
    	   RTE.oneWay <- (rankmean - 0.5)/sum(ind)
	   subject <- factor(subject)
    	   utime <- tlevel
    	   ulength <- length(utime)
    	   SOURCE <- double(ulength)
    	   for (i in 1:ulength) { 		
    		SOURCE[i] <- paste(time.name, utime[i], sep = "")
    	   }	
         out <- data.frame(RankMeans = rankmean, Nobs, RTE.oneWay)
    	   rownames(out) <- SOURCE
         V.oneWay <- (n/(sum(ind)^2)) * covr(mean.time.subj, mind, rankmean, t)

	}else{
		grolen <- length(group)
        	if ((sublen != varlen) || (sublen != grolen)) 
            	stop("Length of the subject does not match the length of group")
	    	glevel <- unique(group)
    	    	a <- length(glevel)
		if (!is.null(group.order)) {
        		glevel <- group.order
        		glevel2 <- unique(group)
        		if (length(glevel) != length(glevel2)) 
            		stop("Length of the group.order vector (", length(glevel), 
                		")\n\t\tis not equal to the levels of group vector (", length(glevel2), ").", sep = "")
        		if (mean(sort(glevel) == sort(glevel2)) != 1) 
            		stop("Elements in the group.order vector is different from the levels specified in the group vector.", sep = "")
    		}	
 	  	newgroup <- double(length(var))

 	   	for (i in 1:length(var)) {
      	  row <- which(subject[i] == slevel)
              col <- which(time[i] == tlevel)
        	  newsubject[i] <- row
        	  newtime[i] <- col
              newgroup[i] <- which(group[i] == glevel)
              sortvector[((col - 1) * s + row)] <- i
   	   	}

	   subject <- newsubject[sortvector]
    	   var <- var[sortvector]
  	   time <- newtime[sortvector]
	   group <- newgroup[sortvector]
         grouptemp <- order(group[1:s])
    	   groupplus <- (rep(c(0:(t - 1)), e = s)) * s
         groupsort <- (rep(grouptemp, t)) + groupplus
    	   subject <- rep(c(1:s), t)

	   var <- var[groupsort]
    	   time <- time[groupsort]
    	   group <- group[groupsort]

    	   group <- factor(group)
   	   time <- factor(time)
   	   subject <- factor(subject)

	   score <- var
    	   N.na <- sum(is.na(score))
   	   ind <- 1 - is.na(score)
	   N <- sum(ind) 			############total de obs sem missing TwoWay
    	   rscore <- rank(score, na.last=TRUE) * ind
         data <- cbind(group, time, subject, rscore, ind)

	   ni <- count.subj(group, subject)
         n <- sum(ni)
  	   tab <- t(matrix(mean.factor(rscore, group:time, ind), t, a))
    	   g.mean <- as.vector(apply(tab, 1, mean, na.rm = TRUE))
    	   t.mean <- as.vector(apply(tab, 2, mean, na.rm = TRUE))
    	   V2 <- V(group, time, subject, rscore, ind, a, t, ni)$V
   	   R <- V(group, time, subject, rscore, ind, a, t, ni)$R
    	   SING.COV <- FALSE
    	   if (qr(V2)$rank < (t * a)) 
        	SING.COV <- TRUE
    	   if (SING.COV) {
        	cat("\n Warning(s):\n")
        	cat(" The covariance matrix is singular. \n")
    	   }
    	   rte <- list(RTE = rte.twoWay(group, time, ind, rscore))
	}

    group_S <- grupo_S
    time_S <- tempo_S
    subject_S <- sujeito_S
    var_S <- variavel_S
    time.order_S <- ordem.tempo_S
    group.order_S <- ordem.grupo_S

    if (is.null(group_S)) {
        group_S <- rep(1, length(subject_S))
    }

    glevel_S <- unique(group_S)
    tlevel_S <- unique(time_S)
    slevel_S <- unique(subject_S)
    t_S <- length(tlevel_S)
    s_S <- length(slevel_S)
    a_S <- length(glevel_S)

    if (!is.null(time.order_S)) {
        tlevel_S <- time.order_S
        tlevel2_S <- unique(time_S)
        if (length(tlevel_S) != length(tlevel2_S)) 
            stop("Length of the time.order vector (", length(tlevel_S), 
                ") \n\t\tis not equal to the levels of time vector (", 
                length(tlevel2_S), ").", sep = "")
        if (mean(sort(tlevel_S) == sort(tlevel2_S)) != 1) 
            stop("Elements in the time.order vector is different from the levels specified in the time vector.", 
                sep = "")
    }

    if (!is.null(group.order_S)) {
        glevel_S <- group.order_S
        glevel2_S <- unique(group_S)
        if (length(glevel_S) != length(glevel2_S)) 
            stop("Length of the group.order vector (", length(glevel_S), 
                ") \n\t\tis not equal to the levels of group vector (", 
                length(glevel2_S), ").", sep = "")
        if (mean(sort(glevel_S) == sort(glevel2_S)) != 1) 
            stop("Elements in the group.order vector is different from the levels specified in the group vector.", 
                sep = "")
    }

    sortvector_S <- double(length(var_S))
    newtime_S <- double(length(var_S))
    newsubject_S <- double(length(var_S))
    newgroup_S <- double(length(var_S))

    for (i in 1:length(var_S)) {
        row_S <- which(subject_S[i] == slevel_S)
        col_S <- which(time_S[i] == tlevel_S)
        newsubject_S[i] <- row_S
        newtime_S[i] <- col_S
        newgroup_S[i] <- which(group_S[i] == glevel_S)
        sortvector_S[((col_S - 1) * s_S + row_S)] <- i
    }

    subject_S <- newsubject_S[sortvector_S]
    var_S <- var_S[sortvector_S]
    time_S <- newtime_S[sortvector_S]
    group_S <- newgroup_S[sortvector_S]
    grouptemp_S <- order(group_S[1:s_S])
    groupplus_S <- (rep(c(0:(t_S - 1)), e = s_S)) * s_S
    groupsort_S <- (rep(grouptemp_S, t_S)) + groupplus_S
    subject_S <- rep(c(1:s_S), t_S)
    var_S <- var_S[groupsort_S]
    time_S <- time_S[groupsort_S]
    group_S <- group_S[groupsort_S]

    timenames_S <- unique(time_S)
    time_S <- factor(time_S)
    subject_S <- factor(subject_S)
    group_S <- factor(group_S)

    facs_S <- levels(group_S)
    A_S <- nlevels(group_S)
    T_S <- nlevels(time_S)

    D <- var_S
    RD <- rank(D,na.last="keep") #acrescentei
    N_S <- length(unique(subject_S))

    Lamda <- 1 - as.numeric(is.na(D))
    Omat <- matrix(NA, N_S, T_S)
    Rmat <- matrix(NA, N_S, T_S)
    Lamdamat <- matrix(NA, N_S, T_S)
    sort1 <- unique(time_S)
    for (i in 1:T_S) {
        tempD <- D[which(time_S == sort1[i])]
        tempRD <- RD[which(time_S == sort1[i])]
        tempLM <- Lamda[which(time_S == sort1[i])]
        tempSJ <- subject_S[which(time_S == sort1[i])]
        orderSJ <- order(tempSJ)
        tempD <- tempD[orderSJ]
        tempRD <- tempRD[orderSJ]
        tempLM <- tempLM[orderSJ]
        if (i == 1) {
            tempFC <- group_S[which(time_S == sort1[i])]
            FAC <- tempFC[orderSJ]
            FAC <- factor(FAC)
        }
        Omat[, i] <- tempD
        Rmat[, i] <- tempRD
        Lamdamat[, i] <- tempLM
    }
    D <- Omat
    NN <- sum(Lamda)
    Rmat <- Rmat * Lamdamat
    RMeans <- rep(0, A_S * T_S)
    Ni <- apply(Lamdamat, 2, sum, na.rm = TRUE)
    Rmat.fac <- cbind(FAC, Rmat)
    colnames(Rmat.fac) <- NULL
    Lamdamat.fac <- cbind(FAC, Lamdamat)
    colnames(Lamdamat.fac) <- NULL
    fn.fact.manip <- function(fullRmat, n, n.a, n.t, A.ni) {
        res.AT <- list(0)
        for (i in 1:n.a) res.AT[[i]] <- fullRmat[(fullRmat[,1] == i), -1]
        return(res.AT)
    }

    Ra.list <- fn.fact.manip(Rmat.fac, N_S, A_S, T_S, as.vector(summary(FAC)))
    R1a.list <- fn.fact.manip(Lamdamat.fac, N_S, A_S, T_S, as.vector(summary(FAC)))

    for (i in 1:A_S) RMeans[((i - 1) * T_S + 1):((i - 1) * T_S + T_S)] <- apply(Ra.list[[i]], 
        2, sum, na.rm=TRUE)/apply(R1a.list[[i]], 2, sum, na.rm=TRUE)

    RTE <- (RMeans - 0.5)/NN
    GROUP <- rep(glevel_S, e = T_S)
    TIME <- rep(tlevel_S, A_S)
    Nobs <- rep(0, A_S * T_S)
    for (i in 1:A_S) Nobs[((i - 1) * T_S + 1):((i - 1) * T_S + T_S)] <- apply(R1a.list[[i]], 
        2, sum,na.rm=TRUE)

    Mean.fac <- cbind(FAC, D)
    Mean.list <- fn.fact.manip(Mean.fac, N_S, A_S, T_S, as.vector(summary(FAC)))
    MediaS<-rep(0, A_S * T_S) 	
    for (i in 1:A_S) MediaS[((i - 1) * T_S + 1):((i - 1) * T_S + T_S)] <- apply(Mean.list[[i]], 
        2, mean, na.rm=TRUE)

    SD.fac <- cbind(FAC, D)
    SD.list <- fn.fact.manip(SD.fac, N_S, A_S, T_S, as.vector(summary(FAC)))
    DP.S<-rep(0, A_S * T_S) 	
    for (i in 1:A_S) DP.S[((i - 1) * T_S + 1):((i - 1) * T_S + T_S)] <- apply(SD.list[[i]], 
        2, sd, na.rm=TRUE)

    PRes1 <- cbind(RMeans, Nobs, RTE, MediaS,DP.S)
    rd.PRes1 <- round(PRes1, Inf)
    rd.PRes1 <- cbind(TIME, rd.PRes1)
    D.fac <- cbind(FAC, D)

    fn.compli.sig <- function(fullDmat, n, nn, n.a, n.t, FAC,A.ni) {
        res.list <- list(0)
        res.R.is <- list(0)
        res.R.mis <- list(0)
        res.bias <- rep(0, 0)
        MAXI <- max(fullDmat)
        for (i in 1:n.a) {
            res.R.is[[i]] <- list(0)
            res.R.mis[[i]] <- list(0)
            temp.Dmat <- fullDmat[(fullDmat[, 1] == i), -1]
            temp.mat <- matrix(0, A.ni[i], n.t)
            temp.Rmat <- matrix(0, A.ni[i], n.t)
            for (j in 1:A.ni[i]) temp.mat[j, ] <- rank(temp.Dmat[j, ],na.last="keep")
            temp.mat <- temp.mat * matrix(1 - as.numeric(is.na(c(temp.mat))), 
                A.ni[i], n.t)
            temp.Rmat <- matrix(rank(temp.Dmat,na.last="keep"), A.ni[i], n.t) * 
                matrix(1 - as.numeric(is.na(c(temp.Dmat))), A.ni[i], 
                  n.t)
            for (s in 1:n.t) {
                res.R.is[[i]][[s]] <- rep(0, 0)
                res.R.mis[[i]][[s]] <- matrix(0, n, n.t + 1)
                temp.Dmat <- fullDmat
                t.r.is <- rank(temp.Dmat[(temp.Dmat[, 1] == i), -1][, s],na.last="keep")
                t.l.is <- 1 - as.numeric(is.na(c(temp.Dmat[(temp.Dmat[, 1] == i), -1][, s])))
                t.r.is <- t.r.is * t.l.is
                temp.Dmat[(temp.Dmat[, 1] == i), -1][, s] <- MAXI + 1
                t1.Rmat <- matrix(rank(temp.Dmat[, -1],na.last="keep"), n, n.t)
                t1.Rmat <- t1.Rmat * matrix(1 - as.numeric(is.na(c(temp.Dmat[,-1]))), n, n.t)
                t1.Rmat[(temp.Dmat[, 1] == i), s] <- 0
                res.R.is[[i]][[s]] <- t.r.is
                res.R.mis[[i]][[s]] <- cbind(FAC, t1.Rmat)
                colnames(res.R.mis[[i]][[s]]) <- NULL
                temp.bias <- (1/A.ni[i]) * (sum(temp.mat[, s],na.rm=TRUE)) - 0.5
                temp.bias <- temp.bias - ((1/A.ni[i]) * ((1/A.ni[i]) * 
                  sum(temp.Rmat[, s],na.rm=TRUE) - 0.5))
                temp.bias <- (A.ni[i]/(nn * (A.ni[i] - 1))) * temp.bias
                res.bias <- c(res.bias, temp.bias)
            }
        }
        res.list[[1]] <- res.R.is
        res.list[[2]] <- res.R.mis
        res.list[[3]] <- res.bias
        return(res.list)
    }
    A.ni <- as.vector(summary(FAC))
    Da.list <- fn.compli.sig(D.fac, N_S, NN, A_S, T_S, FAC, A.ni)

    Y.list <- list(0)
    for (i in 1:A_S) {
        Y.list[[i]] <- matrix(0, A.ni[i], T_S)
        for (k in 1:A.ni[i]) {
            for (s in 1:T_S) {
                Y.list[[i]][k, s] <- (1/NN) * ((2 * Rmat.fac[(Rmat.fac[, 
                  1] == i), -1][k, s]) - Da.list[[1]][[i]][[s]][k] - 
                  sum(Rmat.fac[(Rmat.fac[, 1] == i), -1][k, ],na.rm=TRUE) + 		
                  sum(Da.list[[2]][[i]][[s]][(Da.list[[2]][[i]][[s]][,1] == i),	
				-1][k, ],na.rm=TRUE))
            }
        }
    }
    Z.list <- list(0)
    tau.u.is.sq <- list(0)
    for (i in 1:A_S) {
        Z.list[[i]] <- list(0)
        tau.u.is.sq[[i]] <- list(0)
        for (s in 1:T_S) {
            Z.list[[i]][[s]] <- list(0)
            tau.u.is.sq[[i]][[s]] <- rep(0, A_S)
            for (u in 1:A_S) {
                Z.list[[i]][[s]][[u]] <- rep(0, A.ni[u])
                if (u != i) {
                  for (k in 1:A.ni[u]) {
                    Z.list[[i]][[s]][[u]][k] <- (1/A.ni[i]) * 
                      (sum(Rmat.fac[(Rmat.fac[, 1] == u), -1][k,],na.rm=TRUE) -
					sum(Da.list[[2]][[i]][[s]][(Da.list[[2]][[i]][[s]][,1] == u), -1][k, ],na.rm=TRUE))
                  }
                  tau.u.is.sq[[i]][[s]][u] <- (1/(A.ni[u] - 1)) * 
                    sum((Z.list[[i]][[s]][[u]] - mean(Z.list[[i]][[s]][[u]],na.rm=TRUE)) * 
                      (Z.list[[i]][[s]][[u]] - mean(Z.list[[i]][[s]][[u]],na.rm=TRUE)),na.rm=TRUE)
                }
            }
        }
    }
    lam.is.sq <- matrix(0, A_S, T_S)
    for (i in 1:A_S) {
        for (s in 1:T_S) {
            lam.is.sq[i, s] <- (1/(A.ni[i] - 1)) * 
			(sum((Y.list[[i]][,s] - 
			 mean(Y.list[[i]][, s],na.rm=TRUE)) * (Y.list[[i]][,s] -
			 mean(Y.list[[i]][, s],na.rm=TRUE)),na.rm=TRUE))
        }
    }
    sig.sq <- matrix(0, A_S, T_S)
    for (i in 1:A_S) {
        for (s in 1:T_S) {
            sig.sq[i, s] <- ((N_S/A.ni[i]) * lam.is.sq[i, s]) + 
                ((1/(N_S * T_S * T_S)) * sum(A.ni * tau.u.is.sq[[i]][[s]],na.rm=TRUE))
        }
    }
    P.vec <- t(matrix(RTE, T_S, A_S))  #######Vetor de RTE
    pL <- matrix(0, A_S, T_S)
    pU <- matrix(0, A_S, T_S)
    for (i in 1:A_S) {
        for (s in 1:T_S) {
            pg <- ((NN * P.vec[i, s]) - A.ni[i]/2)/(NN - A.ni[i])
            sd <- (NN * sqrt(sig.sq[i, s]))/(NN - A.ni[i])
            t1 <- log(pg/(1 - pg))
            t2 <- (sd/(pg * (1 - pg) * sqrt(N_S))) * qnorm((1 - 
                (alpha/2)))
            pgL <- t1 - t2
            pgU <- t1 + t2
            pL[i, s] <- (A.ni[i]/(2 * NN)) + ((NN - A.ni[i])/NN) * 
                (exp(pgL)/(1 + exp(pgL)))
            pU[i, s] <- (A.ni[i]/(2 * NN)) + ((NN - A.ni[i])/NN) * 
                (exp(pgU)/(1 + exp(pgU)))
        }
    }
    CI <- cbind(matrix(t(pL), A_S * T_S, 1), matrix(t(pU), A_S * T_S,  1))
    frank.RMeans <- round(RMeans, rounds)
    frank.Nobs <- Nobs
    SMS.Mean <- round(MediaS,rounds)
    SMS.SD <- round(DP.S,rounds)

    frank.RTE <- round(RTE, rounds)
    frank.Bias <- round(c(Da.list[[3]]), rounds)
    frank.Var <- round(c(t(sig.sq)), rounds)
    frank.Lower <- round(matrix(t(pL), A_S * T_S, 1), rounds)
    frank.Upper <- round(matrix(t(pU), A_S * T_S, 1), rounds)
    frank.Time <- TIME
    frank.Group <- paste(GROUP, sep = "")
       
    rd.PRes1 <- cbind(rd.PRes1, round(c(t(sig.sq)),4), round(CI, 4))

    colnames(rd.PRes1) <- c( "Time","RankMeans", "Nobs", "RTE","Means","Standard Deviation", "Variance", "Lower_bound", "Upper_bound")
    rownames(rd.PRes1) <- paste(GROUP, sep = "")
        
    Frank <- data.frame(Group = frank.Group, Time = frank.Time, 
    Nobs = frank.Nobs, Means = SMS.Mean, StDev = SMS.SD, RankMeans = frank.RMeans, RTE = frank.RTE, 
    Variance = frank.Var, Lower = frank.Lower,Upper = frank.Upper)

#*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~
#*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~
#*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~
#Graphics!!!
#primeiro IC RTE
	
if(graphs[3]==TRUE){
	x11()
     	plot.samples <- split(Frank, Frank$Group)
        if (A_S == 1) {
        	plot(1:T_S, plot.samples[[1]]$RTE, pch = 10, type = "b", cex.axis = 1.2, 
            	ylim = c(0, 1), xaxt = "n", xlab = time.name, ylab = "RTE", 
            	cex.lab = 1.2, xlim = c(0, T_S), lwd = 3)
       	 for (l in 1:T_S) {
            	points(rep(l, 2), c(plot.samples[[1]]$Lower[l], plot.samples[[1]]$Upper[l]), 
                		type = "l", lwd = 3)
            	text(l, plot.samples[[1]]$Lower[l]+0.00175, "_", cex = 2)
            	text(l, plot.samples[[1]]$Upper[l]+0.00185, "_", cex = 2)
        	}
        axis(1.2, at = 1:T_S, labels = plot.samples[[1]]$Time, cex.axis = 1.2)
        }
        if (A_S >= 2){
        plot(1:T_S, plot.samples[[1]]$RTE, pch = 10, type = "b", cex.axis = 1.2,
            ylim = c(0, 1), xaxt = "n", xlab = time.name, ylab = "RTE", 
            cex.lab = 1.2, xlim = c(0, (T_S + 2.5)), lwd = 3)
        for (l in 1:T_S) {
            points(rep(l, 2), c(plot.samples[[1]]$Lower[l], plot.samples[[1]]$Upper[l]), 
                type = "l", lwd = 3)
            text(l, plot.samples[[1]]$Lower[l]+0.00175, "_", cex = 2)
            text(l, plot.samples[[1]]$Upper[l]+0.00185, "_", cex = 2)
        }
         for (kn in 2:A_S) {
                for (fk in 1:T_S) {
                  x.neu <- c(1:T_S)
                  points(x.neu, plot.samples[[kn]]$RTE, pch = 10, 
                    type = "b", lwd = 2, ylim = c(0, 1.1), xaxt = "n", 
                    xlab = "", ylab = "", cex.lab = 1.5, col = kn)
                  points(rep(x.neu[fk], 2), c(plot.samples[[kn]]$Lower[fk], 
                    plot.samples[[kn]]$Upper[fk]), type = "l", 
                    col = kn, lwd = 3)

                  text(x.neu[fk], plot.samples[[kn]]$Lower[fk]+0.00175, 
                    "_", cex = 2, col = kn)
                  text(x.neu[fk], plot.samples[[kn]]$Upper[fk]+0.0018, 
                    "_", cex = 2, col = kn)
                }
            }
            axis(1.2, at = 1:T_S, labels = plot.samples[[1]]$Time, cex.axis = 1.2)
            namen.g.plot1 <- seq(1, A_S * T_S, by = T_S)
            namen.g.plot2 <- paste(Frank$Group[namen.g.plot1])
            legend("topright",c((T_S + .5), 1), col = c(1:a_S), namen.g.plot2, pch = c(rep(10, a_S)), lwd = c(rep(3, a_S)), bty="n")
        }
#dev.off()
}


#segundo IC media

if(graphs[2]==TRUE){
	x11()
            namen.g.plot1 <- seq(1, A_S * T_S, by = T_S)
            namen.g.plot2 <- paste(Frank$Group[namen.g.plot1])

		if(!is.null(grupo_S) && !is.null(emptyCell)){		
			lineplot.CI(tempo_S, variavel_S, group = grupo_S, cex=1.3, lwd=3, legend=FALSE, 
				xlab = time.name, ylab = var.name, cex.lab = 1.2, cex.axis=1.2, xlim = c(0, T_S+3),
				pch = rep(16,A_S), err.width=0.1, err.lty = 1, col=1:A_S, lty=1,
				ci.fun= function(x) c(mean(x, na.rm=TRUE)-1.96*se(x, na.rm=TRUE), mean(x, na.rm=TRUE)+1.96*se(x, na.rm=TRUE)),fixed=TRUE)
        			legend("topright",c((T_S + .5), 1), col = c(1:a_S), namen.g.plot2, pch = c(rep(10, a_S)), lwd = c(rep(3, a_S)), bty="n")

		}

		if(!is.null(grupo_S) && is.null(emptyCell)){	
			xx<-data.frame(variavel_S, tempo_S, grupo_S, sujeito_S)
			with(xx, {
				## order the rows by their mean effect
				tempo_S2<- factor(tempo_S,levels = time.order)
				lineplot.CI(tempo_S2, variavel_S, group = grupo_S, cex=1.3, lwd=3, legend=FALSE,
					xlab = time.name, ylab = var.name, cex.lab = 1.2, cex.axis=1.2, xlim = c(0, T_S+3),
					pch = rep(16,A_S), err.width=0.1, err.lty = 1, col=1:A_S, lty=1,
					ci.fun= function(x) c(mean(x, na.rm=TRUE)-1.96*se(x, na.rm=TRUE), mean(x, na.rm=TRUE)+1.96*se(x, na.rm=TRUE)),fixed=TRUE)
        				legend("topright",c((T_S + .5), 1), col = c(1:a_S), namen.g.plot2, pch = c(rep(10, a_S)), lwd = c(rep(3, a_S)), bty="n")
			})

		}
		if(is.null(grupo_S) && is.null(emptyCell)){
			xx<-data.frame(variavel_S, tempo_S, sujeito_S)
			with(xx, {
				## order the rows by their mean effect
				tempo_S2<- factor(tempo_S,levels = time.order)
				lineplot.CI(tempo_S2, variavel_S, group = NULL, cex =1.3, lwd=3, legend=FALSE, 
					xlab = time.name, ylab = var.name, cex.lab = 1.2, cex.axis=1.2, xlim = c(0, T_S),
					pch = 16, err.width=0.1, err.lty = 1, col=1, lty=1, 
					ci.fun= function(x)  c(mean(x, na.rm=TRUE)-1.96*se(x, na.rm=TRUE), mean(x, na.rm=TRUE)+1.96*se(x, na.rm=TRUE)))
			})
		}
#dev.off()
}

#terceiro perfis individuais

if(graphs[1]==TRUE){	
#png(file = paste("p_individual", var.name,".png"))
	x11()
	g2<-as.integer(A_S/2 + 0.5)

		if(!is.null(grupo_S)){
			xx<-data.frame(variavel_S, tempo_S, grupo_S, sujeito_S)
			ymin<-min(xx$variavel_S, na.rm =TRUE)
			ymax<-max(xx$variavel_S, na.rm =TRUE)
			par(mfrow=c(g2,2))

			if(!is.null(emptyCell)){
				for(i in 1:A_S){
					interaction.plot(xx$tempo_S[xx$grupo_S==group.order_S[i]],
						xx$sujeito_S[xx$grupo_S==group.order_S[i]],
						xx$variavel_S[xx$grupo_S==group.order_S[i]], main=group.order_S[i],
						xlab = time.name, ylab = var.name, type="l", legend=FALSE, cex.lab = 1.2, cex.axis=1.2,
						ylim = c(ymin,ymax))			
				}
			}else{
				for(i in 1:A_S){
					with(xx, {
						## order the rows by their mean effect
						xx$tempo_S2<- factor(tempo_S,levels = time.order)
						interaction.plot(xx$tempo_S2[grupo_S==group.order_S[i]],
							xx$sujeito_S[grupo_S==group.order_S[i]],
							xx$variavel_S[grupo_S==group.order_S[i]], main=group.order_S[i],
							xlab = time.name, ylab = var.name, type="l", legend=FALSE, cex.lab = 1.2, cex.axis = 1.2,
							ylim = c(ymin,ymax))	
						})
				}
			}
		  }else{
				xx<-data.frame(variavel_S, tempo_S, sujeito_S)
				with(xx, {
				## order the rows by their mean effect
					tempo_S<- factor(tempo_S,levels = time.order)
					interaction.plot(tempo_S,sujeito_S,variavel_S, #main="Individual Plot",
					xlab = time.name, ylab = var.name, type="l", legend=FALSE, cex.lab = 1.2, cex.axis = 1.2,
					ylim = range(xx$variavel_S, na.rm=TRUE))
				})
		}
#dev.off()
}
   wald_ANOVA.test <- function(RTE, t, n, V, C) {
			 CVC <- C %*% V %*% t(C)  
      		 p <- matrix(RTE, t, 1)
      		 cp <- C %*% p
		       Q <- n * t(cp) %*% ginv(CVC) %*% cp
		       dfq <- sum(diag(CVC %*% ginv(CVC)))
		       if ((!is.na(Q)) && (!is.na(dfq)) && (Q > 0) && (dfq > 0)) 
           			pvQ <- 1 - pchisq(Q, dfq)
     			 else pvQ <- NA
 		       wald.test<- data.frame(Statistic = round(Q,3), df = round(dfq,2), P.value = round(pvQ,rounds))
   		       colnames(wald.test) <- c("Statistic", "df", "p-value")
     		       rownames(wald.test) <- time.name
      		 T <- t(C) %*% ginv(C %*% t(C)) %*% C
      		 f <- (sum(diag(T %*% V)))^2/sum(diag(T %*% V %*% T %*% V))
      		 A <- n * t(p) %*% T %*% p/sum(diag(T %*% V))
      		 if ((!is.na(A)) && (!is.na(f)) && (A > 0) && (f > 0)) 
      		     pvA <- 1 - pchisq(A * f, f)
      		 else pvA <- NA
      		 anova.test<- data.frame(Statistic = round(A,3), df = round(f,2), P.value = round(pvA,rounds))
      		 colnames(anova.test) <- c("Statistic", "df", "p-value")
      		 rownames(anova.test) <- time.name
			 out <- list(Contrast=round(C,2), Wald.test=wald.test, ANOVA.test=anova.test)
			 return(out)
	}
#~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.
# FUNCAO QUE CRIA A MATRIZ BLOCO DIAGONAL!!!
#~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.
	Bdiag <- function(M,g,t, k){
		aux <- matrix(0,g*t,g*t)
		#k<-which(is.na(M[1,]))
		for(i in 1:g){
			aux[((i-1)*t+1):(i*t),((i-1)*t+1):(i*t)]<- M[((i-1)*t+1):(i*t),((i-1)*t+1):(i*t)]
		}
		aux[k,]<-NA
		aux[,k]<-NA
		return(aux)
	}
#~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.
#	Testes 
#~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.
	if(is.null(group)){ #one-way ou celula vazia
		if(!is.null(emptyCell)){		#CLULA VAZIA!!!!
		    	NA.obs<-which(t(Frank$Nobs)==0)
			Frank_EC<-Frank[-NA.obs,]

			if(is.null(contrast)){stop(paste(" necessrio declarar a(s) matriz(es) de contraste(s)!" ,sep=""))}
			p <- matrix(RTE.oneWay, t, 1)
#~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.
#	Retira as clulas vazias do vetor de RTE
#~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.
	    	 	p.na <- 1-is.na(p)
			lin<-numeric(t)
		 	for(i in 1:t){
				if(p.na[i,]==0){lin[i]<-i
				}else{lin[i]<-0}
		 	}
		 	if(sum(lin)==0){
		 		p<-p
		 	}else{
				p<-p[-lin,] #matriz de RTE sem as linhas vazias
		 	}

			V.BDiag <- Bdiag(M=V.oneWay,g=gg,t=tt, k=lin) #transforma a matriz de var/cov em bloco diag.
			V.EC <- V.BDiag[-lin,-lin] 		#retiras as linhas/colunas vazias

			r_rows <- 0 
			c_cols <- 0 
			RTE<-p
	
			Ctam <- length(contrast)
			Stat<-list()
			for(i in 1:Ctam){
				r_rows[i] <- nrow(contrast[[i]]) 
				c_cols[i] <- ncol(contrast[[i]]) 
				for(i in 1:Ctam){ 	
					Stat[[i]] <- wald_ANOVA.test(RTE=RTE, t=length(RTE), n=n, V=V.EC, C=contrast[[i]])	 
				}
				EC.RTE <- data.frame(Names=time.order,RTE=round(RTE.oneWay,rounds))
				EC.RTE<-EC.RTE[-lin,]
				Stat <- list(Tests=Stat)
			}
	
		}else{		#oneway!!!!!!
			out1 <- data.frame(RankMeans = rankmean, Nobs, RTE.oneWay)
    			rownames(out1) <- SOURCE
			V.oneWay <- (n/(sum(ind)^2)) * covr(mean.time.subj, mind, rankmean, t)
			wald.test <- NULL
			anova.test <- NULL
			RTE<-RTE.oneWay

			if (t > 1){ 	
				C <- diag(t) - matrix(1/t, t, t)
				wald_ANOVA.test.OW <- wald_ANOVA.test(RTE.oneWay, t, n, V.oneWay, C) 
				hy <- "Usuais Null Hypothesis Tests"
				out  <- list(hy, Wald_ANOVA.test = wald_ANOVA.test.OW)
				Covariance = round(V.oneWay,rounds)
			}	
			if(!is.null(contrast)){ 
				Ctam <- length(contrast)
				Stat<-list()
				for(i in 1:Ctam){
					Stat[[i]] <- wald_ANOVA.test(RTE=RTE, t=length(RTE), n=n, V=V.oneWay, C=contrast[[i]])	 
				}
				EC.RTE <- data.frame(Names=time.order,RTE=round(RTE.oneWay,rounds))
				Stat <- list(Tests=Stat)
			}
		}
	}else{	#twoWay !!!!!!!!!!!!!!
  		wald.test.t <- wald.test.twoWay(group, time, subject, rscore, ind, ni, rounds)
		anova.test.t <- anova.test.twoWay(group, time, subject, rscore, ind, a, ni, rounds)
		out <- list(Wald.test = wald.test.t, ANOVA.test = anova.test.t)
		Covariance = round(V2,2)
		ERT<-t(frank.RTE)
#~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.
#	Testa se  possvel testar os contrates definidos pelo usurio
#~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.~.
		if(!is.null(contrast)){ 
			r_rows <- 0 
			c_cols <- 0 
			Ctam <- length(contrast)
			for(i in 1:Ctam){
				r_rows[i] <- nrow(contrast[[i]]) 
				c_cols[i] <- ncol(contrast[[i]]) 
				if(c_cols[i] != a*t){stop(paste("Number of columns wrong!" ,sep=""))}

				Stat<-list()
				for(i in 1:Ctam){ 	
					Stat[[i]] <- Statistics(C=round(contrast[[i]],2), V=V2, p=ERT,n=n, g=r_rows[i])		
				}
				Stat <- list(Tests=Stat)
			}
		} 	  
	}
	cat("\nNONPARAMETRIC ANALYSES OF REPEATED DATA.","\n",sep="")
	cat("\n",sep="")
	cat("\nNonparametric Estimates of Relative Treatment Effects.","\n",sep="")
	cat("\n")
  if(!is.null(emptyCell)){
	cat("\nEmpty Cell - Contrasts specified by the user.","\n",sep="")
	cat("\n")
		return(list(Summary=Frank_EC, Stat=Stat, Covariance = round(V.EC,3)))
	}else{
		if(!is.null(contrast)){
			return(list(summary=Frank,Test=out,Stat=Stat,Covariance=Covariance))	
		}else{
			return(list(summary=Frank,Test=out,Covariance=Covariance))	
		}
	}	 

}